package functionalDependency.general;

/**
 * Die Klasse <code>Main</code> kann als einfache Test- und Hauptklasse
 * benutzt werden.
 * <p>
 * Implementiert ist die Methode {@link #main(String[])}, die den
 * <code>String</code> <code>INPUT</code> einliest und fr die
 * gegebene Attributmenge und Menge funktionaler Abhaengigkeiten
 * </p>
 * <ul>
 * <li>alle Schluesselkandidaten,
 * <li>alle Schluessel- und Nichtschluesselattribute,
 * <li>die Basis,
 * <li>die Normalform,
 * <li>eine Zerlegung mittels Synthese und
 * <li>eine Zerlegung mittels Dekomposition
 * </ul>
 * <p>
 * berechnet. Die Ausgabe erfolgt auf der Konsole.
 * </p>
 * <p>
 * Im Rahmen der Aufgabenstellung muss in dieser Klasse nichts geaendert werden.
 * </p>
 */
public class Main {
	
	/**
	 * Die Eingabe des Programms.
	 * <ul>
	 * <li>1. Spalte (<code>INPUT_ATTR</code>): Attributmenge
	 * <li>2. Spalte (<code>INPUT_FDS</code>): Menge von FDs
	 * </ul>
	 */
	private static final String[][] INPUT = {
		// INPUT_ATTR	INPUT_FDS
		{"{ABCDE}",		"{ABC->DE;BCD->AE;D->E}"},
		{"{ABCDEF}",	"{AB->CDEF;E->F}"},
		{"{ABCDE}",		"{ABC->DE;BCD->AE}"},
		{"{ABCDE}",		"{ABC->DE;D->C;D->E}"},
		{"{ABCDEF}",	"{A->D;AB->CDEF;E->F}"},
		{"{ABCDEF}",	"{A->D;AB->CDEF;D->B;E->F}"},
		{"{ABCDE}",		"{ABC->DE;BCD->AE}"},
		{"{ABCDE}",		"{ABC->DE;BCD->AE;C->D}"},
		{"{ABCDEF}",	"{AB->CD;ABDE->F;BC->A;C->DF}"},
		{"{ABCDEFGH}",	"{A->BCGH;BC->EGH;BE->DFH}"},
		{"{ABCDE}", 	"{ABC->D;BCD->A;D->E}"},
		{"{ABCDEF}",	"{AB->C;C->B}"},	
		{"{ABC}",		"{A->BC;BC->A}"},
	};

	// Input-Array
	// -----------
	
	/** Position der Attributmenge im <code>INPUT</code>-Array. */
	protected static final int INPUT_ATTR = 0;
	/** Position der funktionalen Abhaengigkeiten im <code>INPUT</code>-Array. */
	protected static final int INPUT_FDS = 1;
	
	/**
	 * Ausgabe
	 * <ul>
	 * <li>aller Schluesselkandidaten,
	 * <li>aller Schluessel- und Nichtschluesselattribute,
	 * <li>der Basis,
	 * <li>der Normalform,
	 * <li>einer Zerlegung mittels Synthese,
	 * <li>einer Zerlegung mittels Dekomposition
	 * </ul>
	 * der Schemata im <code>INPUT</code>-Array.
	 */
	public static void main(String[] args) throws MainException {
		for (int i = 0; i < INPUT.length; i++) {
			AttributeSet attributes = AttributeSet.parseSchemaFromLine(INPUT[i][INPUT_ATTR]);
			SetOfFDs fds = SetOfFDs.parseSchemaFromLine(INPUT[i][INPUT_FDS]);
			RelationSchema rs = new RelationSchema("R", attributes, fds);
			RelationalDatabaseSchema rds = new RelationalDatabaseSchema(attributes, fds);
			rds.addSchema(rs);
			System.out.println("----------\n"
					+ (i + 1) + ".\n"
					+ "----------\n\n"
					+ "Attributmenge = " + attributes + "\n"
					+ "FDs = " + fds + "\n"
//					+ "RelationSchema rs: " + rs + "\n"
//					+ "RelationalDatabaseSchema rds:\n" + rds
					+ "\n"
					+ "- alle Schluesselkandidaten: " + rs.getKeys() + "\n"
					+ "- Schluesselattribute: " + rs.getPrimeAttributeSet() + "\n"
					+ "- Nichtschluesselattribute: " + rs.getNonPrimeAttributeSet() + "\n"
					+ "- Basis: " + fds.base() + "\n"
					+ "- Normalform: " + rs.getNormalForm() + "\n"
					+ "- 3NF-Schema (Synthese):\n" + rds.convertInto3NF()
					+ "- BCNF-Schema (Dekomposition):\n" + rds.convertIntoBCNF());
		}
	}
}
