package functionalDependency.general;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

/**
 * Die abstrakte Klasse <code>SetOf</code> stellt Methoden fuer die Bildung von
 * Mengen zur Verfuegung. Sie wird von <code>{@link SetOfAttributeSets}</code>, 
 * <code>{@link SetOfFDs}</code> und <code>{@link SetOfRelationSchemata}</code>
 * verwendet. Die jeweiligen Mengen werden in einer Baumstruktur in der
 * <code>TreeSet</code>-Variablen <code>setVar</code> gespeichert.
 * <p>Im Rahmen der Aufgabenstellung muss in dieser Klasse nichts geaendert werden.</p>
 */
public abstract class SetOf<TYPE> implements Set<TYPE>,Cloneable{
	
	TreeSet<TYPE> setVar;
	static final long serialVersionUID = 10L;
	
	private String delimiter;
	
	public SetOf() {
		super();
		delimiter = ";";
		setVar = new TreeSet<TYPE>();
	}
	
	public SetOf(String del) {
		super();
		delimiter = del;
		setVar = new TreeSet<TYPE>();
	}
	
	public TYPE first(){
		TYPE rc = null;
		Iterator<TYPE> i = this.iterator();
		if(i.hasNext())
			rc = i.next();
		return rc;
	}
	
	public String getDelimiter(){
		return delimiter;
	}
	
	public void setDelimiter(String d){
		delimiter = d;
	}
	
	public int size() {
		return setVar.size();
	}

	public boolean isEmpty() {
		return setVar.isEmpty();
	}

	public boolean contains(Object arg) {
		return setVar.contains(arg);
	}

	public Iterator<TYPE> iterator() {
		return setVar.iterator();
	}

	public boolean add(TYPE arg){
		return setVar.add(arg);
	}

	public boolean remove(Object arg) {
		return setVar.remove(arg);
	}

	public boolean containsAll(Collection<?> arg) {
		return setVar.containsAll(arg);
	}

	public boolean addAll(Collection<? extends TYPE> arg) {
		return setVar.addAll(arg);
	}

	public boolean retainAll(Collection<?> arg) {
		return setVar.retainAll(arg);
	}

	public boolean removeAll(Collection<?> arg) {
		return setVar.removeAll(arg);
	}

	public void clear() {
		setVar.clear();
	}
	
	abstract public SetOf<TYPE> clone();
	
	public String toString(){
		boolean flag = false;
		String rc = "{";
		for(TYPE e : setVar){
			if(flag)
				rc += delimiter;
			rc += e.toString();
			flag = true;
		}
		rc += "}";
		return rc;
	}
	
	public boolean equals(SetOf<TYPE> s){
		return setVar.equals(s.setVar);
	}

	public boolean equals(Object o){
		if(o instanceof SetOf)
			return equals((SetOf<TYPE>)o);
		return false;
	}

	public Object[] toArray() {
		return setVar.toArray();
	}

	public Object[] toArray(Object[] arg) {
		return setVar.toArray(arg);
	}
	
}
