package functionalDependency.general;

import java.io.StringReader;
import java.util.TreeSet;

import functionalDependency.parser.Parser;

/**
 * Die Klasse <code>SetOfAttributeSets</code> repraesentiert eine Menge von
 * Attributmengen. Sie ist von der abstrakten Klasse {@link SetOf} abgeleitet.
 * <p>Implementiert sind neben dem Konstruktor die Methoden</p>
 * <ul>
 * <li> {@link #parseSchemaFromLine(String)}: Parsen eines Strings
 * <li> {@link #clone()}: Erstellen einer Kopie der Menge von Attributmengen
 * <li> verschiedene Teil- und Obermengentests
 * </ul>
 * <p>Im Rahmen der Aufgabenstellung muss in dieser Klasse nichts geaendert werden.</p>
 */
public class SetOfAttributeSets extends SetOf<AttributeSet> {

	public SetOfAttributeSets() {
		super(";");
	}

	static public SetOfAttributeSets parseSchemaFromLine(String line) throws MainException {
		Parser parser = null;
		StringReader reader;
		reader = new StringReader(line);
		parser = new Parser(reader);
		return parser.parseAttrSetDef();
	}
	
	public SetOfAttributeSets clone(){
		SetOfAttributeSets rc = new SetOfAttributeSets();
		rc.setVar = (TreeSet<AttributeSet>)this.setVar.clone();
		return rc;
	}
	
	/**
	 * Prueft, ob ein Element der Menge von Attributenmengen
	 * (= <code>AttributeSet</code>) eine Teilmenge von <code>a</code> ist.
	 * @param a die zu testende Attributmenge
	 * @return <code>true</code>, wenn die Menge von Attributemengen eine Teilmenge
	 * von <code>a</code> enthaelt; <code>false</code>, andernfalls
	 */
	public boolean containsSubSetFor(AttributeSet a) {
		for (AttributeSet e : setVar)
			if (a.includes(e))
				// a ist Obermenge von e
				return true;
		return false;
	}

	/**
	 * Wie {@link #containsSubSetFor(AttributeSet)} mit Test auf echter Teilmenge
	 * @param a die zu testende Attributmenge
	 * @return <code>true</code>, wenn die Menge von Attributemengen eine echte Teilmenge
	 * von <code>a</code> enthaelt; <code>false</code>, andernfalls
	 */
	public boolean containsProperSubSetFor(AttributeSet a) {
		for(AttributeSet e : setVar)
			if(a.includes(e) && !a.equals(e))
				// a ist echte Obermenge von e
				return true;
		return false;
	}
	
	/**
	 * Wie {@link #containsSubSetFor(AttributeSet)} mit Test auf Obermenge
	 * @param a die zu testende Attributmenge
	 * @return <code>true</code>, wenn die Menge von Attributemengen eine Obermenge
	 * von <code>a</code> enthaelt; <code>false</code>, andernfalls
	 */
	public boolean containsSuperSetOf(AttributeSet a){
		for(AttributeSet e : setVar)
			if(e.includes(a))
				// a ist Teilmenge von e
				return true;
		return false;
	}

	/**
	 * Wie {@link #containsSubSetFor(AttributeSet)} mit Test auf echter Obermenge
	 * @param a die zu testende Attributmenge
	 * @return <code>true</code>, wenn die Menge von Attributemengen eine echte 
	 * Obermenge von <code>a</code> enthaelt; <code>false</code>, andernfalls
	 */
	public boolean containsProperSuperSetOf(AttributeSet a){
		for (AttributeSet e : setVar)
			if (e.includes(a) && !a.equals(e))
				// a ist echte Teilmenge von e
				return true;
		return false;
	}
	
	/**
	 * Vereinigt zwei Mengen von <code>AttributeSet<code>
	 * 
	 * @param setattributesets (SetOfAttributeSets)
	 * @return Vereinigung mit dieser Menge (this)
	 */
	public SetOfAttributeSets union(SetOfAttributeSets setofattributesets) {
		SetOfAttributeSets clone = this.clone();
		while (!setofattributesets.isEmpty()) {
			AttributeSet toadd = setofattributesets.first();
			if (!clone.contains(toadd)) {
				clone.add(toadd);
			}
			setofattributesets.remove(setofattributesets.first());
		}
		return clone;
	}
	
}
